
From: Hugh Dickins <hugh@veritas.com>

Fix "scheduling while atomic" messages below do_anonymous_page when
CONFIG_HIGHPTE=y: must unmap and remap the page_table around page
allocation.  And let's shift the usual pte_unmap to the minor_fault exit.

Signed-off-by: Hugh Dickins <hugh@veritas.com>
Cc: Christoph Lameter <christoph@lameter.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 mm/memory.c |    6 +++---
 1 files changed, 3 insertions(+), 3 deletions(-)

diff -puN mm/memory.c~page-fault-patches-fix-highpte-in-do_anon_page mm/memory.c
--- devel/mm/memory.c~page-fault-patches-fix-highpte-in-do_anon_page	2005-07-30 13:20:29.000000000 -0700
+++ devel-akpm/mm/memory.c	2005-07-30 13:20:29.000000000 -0700
@@ -1761,12 +1761,12 @@ do_anonymous_page(struct mm_struct *mm, 
 		} else {
 			inc_page_state(cmpxchg_fail_anon_read);
 		}
-		pte_unmap(page_table);
 		goto minor_fault;
 	}
 
 	/* This leaves the write case */
 	page_table_atomic_stop(mm);
+	pte_unmap(page_table);
 	if (unlikely(anon_vma_prepare(vma)))
 		goto oom;
 
@@ -1777,10 +1777,10 @@ do_anonymous_page(struct mm_struct *mm, 
 	entry = maybe_mkwrite(pte_mkdirty(mk_pte(page,
 						vma->vm_page_prot)),
 				vma);
+	page_table = pte_offset_map(pmd, addr);
 	page_table_atomic_start(mm);
 
 	if (!ptep_cmpxchg(mm, addr, page_table, orig_entry, entry)) {
-		pte_unmap(page_table);
 		page_cache_release(page);
 		inc_page_state(cmpxchg_fail_anon_write);
 		goto minor_fault;
@@ -1794,12 +1794,12 @@ do_anonymous_page(struct mm_struct *mm, 
 	page_add_anon_rmap(page, vma, addr);
 	lru_cache_add_active(page);
 	inc_mm_counter(mm, rss);
-	pte_unmap(page_table);
 	update_mmu_cache(vma, addr, entry);
 	lazy_mmu_prot_update(entry);
 
 minor_fault:
 	page_table_atomic_stop(mm);
+	pte_unmap(page_table);
 	return VM_FAULT_MINOR;
 
 oom:
_
